@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
-""" Canonical Interval Forest Classifier (CIF).
-"""
+"""Canonical Interval Forest Classifier (CIF)."""
 
 __author__ = ["Matthew Middlehurst"]
 __all__ = ["CanonicalIntervalForest"]
@@ -9,22 +8,23 @@ import numpy as np
 import math
 
 from joblib import Parallel, delayed
-from sklearn.ensemble._forest import ForestClassifier
-from sklearn.tree import DecisionTreeClassifier
 from sklearn import clone
+from sklearn.base import BaseEstimator
+from sklearn.tree import DecisionTreeClassifier
 from sklearn.utils import check_random_state
 from sklearn.utils.multiclass import class_distribution
 
-from sktime.utils.validation._dependencies import _check_soft_dependencies
-from sktime.utils.slope_and_trend import _slope
+
+from sktime.contrib._continuous_interval_tree import (
+    _cif_feature,
+    ContinuousIntervalTree,
+)
+from sktime.transformations.panel.catch22 import Catch22
 from sktime.utils.validation.panel import check_X, check_X_y
 from sktime.classification.base import BaseClassifier
 
-_check_soft_dependencies("catch22")
-from sktime.transformations.panel.catch22_features import Catch22  # noqa: E402
 
-
-class CanonicalIntervalForest(ForestClassifier, BaseClassifier):
+class CanonicalIntervalForest(BaseClassifier):
     """Canonical Interval Forest Classifier.
 
     Interval based forest making use of the catch22 feature set on randomly
@@ -41,14 +41,14 @@ class CanonicalIntervalForest(ForestClassifier, BaseClassifier):
     ensemble the trees with averaged probability estimates
 
     This implementation deviates from the original in minor ways. Predictions
-    are made using summed probabilites instead of majority vote
+    are made using summed probabilities instead of majority vote
     and it does not use the splitting criteria tiny refinement described in
-    deng13forest.
+    deng13forest by default.
 
     Parameters
     ----------
-    n_estimators       : int, ensemble size, optional (default to 500)
-    n_intervals         : int, number of intervals to extract, optional (default to
+    n_estimators       : int, ensemble size, optional (default to 200)
+    n_intervals        : int, number of intervals to extract, optional (default to
     sqrt(series_length)*sqrt(n_dims))
     att_subsample_size : int, number of catch22/tsf attributes to subsample
     per classifier, optional (default to 8)
@@ -92,22 +92,22 @@ class CanonicalIntervalForest(ForestClassifier, BaseClassifier):
         "multivariate": True,
         "unequal_length": False,
         "missing_values": False,
+        "train_estimate": False,
+        "contractable": False,
     }
 
     def __init__(
         self,
         min_interval=3,
         max_interval=None,
-        n_estimators=500,
+        n_estimators=200,
         n_intervals=None,
         att_subsample_size=8,
+        base_estimator=None,
         n_jobs=1,
         random_state=None,
     ):
-        super(CanonicalIntervalForest, self).__init__(
-            base_estimator=DecisionTreeClassifier(criterion="entropy"),
-            n_estimators=n_estimators,
-        )
+        self.base_estimator = base_estimator
 
         self.n_estimators = n_estimators
         self.n_intervals = n_intervals
@@ -130,21 +130,20 @@ class CanonicalIntervalForest(ForestClassifier, BaseClassifier):
         self.intervals = []
         self.dims = []
         self.classes_ = []
+        self.tree = None
 
-        # We need to add is-fitted state when inheriting from scikit-learn
-        self._is_fitted = False
+        super(CanonicalIntervalForest, self).__init__()
 
     def fit(self, X, y):
-        """Build a forest of trees from the training set (X, y) using random
-        intervals and catch22/tsf summary features
+        """Build a forest of trees from the training set (X, y).
+
+         Uses random intervals and catch22/tsf summary features.
 
         Parameters
         ----------
-        X : array-like or sparse matrix of shape = [n_instances,
+        X : array-like or sparse matrix of shape = [n_instances,n_dimensions,
         series_length] or shape = [n_instances,series_length]
-            The training input samples.  If a Pandas data frame is passed it
-            must have a single column (i.e. univariate
-            classification).
+        The training input samples.
         y : array-like, shape =  [n_instances]    The class labels.
 
         Returns
@@ -157,12 +156,22 @@ class CanonicalIntervalForest(ForestClassifier, BaseClassifier):
         self.n_classes = np.unique(y).shape[0]
         self.classes_ = class_distribution(np.asarray(y).reshape(-1, 1))[0][0]
 
+        if self.base_estimator is None or self.base_estimator == "DTC":
+            self.tree = DecisionTreeClassifier(criterion="entropy")
+        elif self.base_estimator == "CIT":
+            self.tree = ContinuousIntervalTree()
+        elif isinstance(self.base_estimator, BaseEstimator):
+            self.tree = self.base_estimator
+        else:
+            raise ValueError("DrCIF invalid base estimator given.")
+
         if self.n_intervals is None:
             self.__n_intervals = int(
                 math.sqrt(self.series_length) * math.sqrt(self.n_dims)
             )
         if self.__n_intervals <= 0:
             self.__n_intervals = 1
+
         if self.series_length < self.min_interval:
             self.min_interval = self.series_length
 
@@ -186,15 +195,12 @@ class CanonicalIntervalForest(ForestClassifier, BaseClassifier):
         return self
 
     def predict(self, X):
-        """
-        Find predictions for all cases in X. Built on top of predict_proba
+        """Predict for all cases in X. Built on top of predict_proba.
+
         Parameters
         ----------
-        X : The training input samples. array-like or pandas data frame.
-        If a Pandas data frame is passed, a check is performed that it only
-        has one column.
-        If not, an exception is thrown, since this classifier does not yet have
-        multivariate capability.
+        X : The training input samples. array-like or sparse matrix of shape
+        = [n_test_instances,n_dimensions,series_length]
 
         Returns
         -------
@@ -209,23 +215,18 @@ class CanonicalIntervalForest(ForestClassifier, BaseClassifier):
         )
 
     def predict_proba(self, X):
-        """
-        Find probability estimates for each class for all cases in X.
+        """Probability estimates for each class for all cases in X.
+
         Parameters
         ----------
         X : The training input samples. array-like or sparse matrix of shape
-        = [n_test_instances, series_length]
-            If a Pandas data frame is passed (sktime format) a check is
-            performed that it only has one column.
-            If not, an exception is thrown, since this classifier does not
-            yet have
-            multivariate capability.
+        = [n_test_instances,n_dimensions,series_length]
 
         Local variables
         ----------
         n_test_instances     : int, number of cases to classify
         series_length    : int, number of attributes in X, must match
-        _num_atts determined in fit
+        series_length determined in fit
 
         Returns
         -------
@@ -249,7 +250,6 @@ class CanonicalIntervalForest(ForestClassifier, BaseClassifier):
                 self.intervals[i],
                 self.dims[i],
                 self.atts[i],
-                n_test_instances,
             )
             for i in range(self.n_estimators)
         )
@@ -260,8 +260,8 @@ class CanonicalIntervalForest(ForestClassifier, BaseClassifier):
         return output
 
     def _fit_estimator(self, X, y, idx):
-        c22 = Catch22()
-        rs = 5465 if self.random_state == 0 else self.random_state
+        c22 = Catch22(outlier_norm=True)
+        rs = 255 if self.random_state == 0 else self.random_state
         rs = None if self.random_state is None else rs * 37 * (idx + 1)
         rng = check_random_state(rs)
 
@@ -301,52 +301,59 @@ class CanonicalIntervalForest(ForestClassifier, BaseClassifier):
                 intervals[j][0] = intervals[j][1] - length
 
             for a in range(0, self.att_subsample_size):
-                transformed_x[self.att_subsample_size * j + a] = self.__cif_feature(
+                transformed_x[self.att_subsample_size * j + a] = _cif_feature(
                     X, intervals[j], dims[j], atts[a], c22
                 )
 
-        tree = clone(self.base_estimator)
+        tree = clone(self.tree)
         tree.set_params(random_state=rs)
         transformed_x = transformed_x.T
-        np.nan_to_num(transformed_x, False, 0, 0, 0)
+        transformed_x = transformed_x.round(8)
+        transformed_x = np.nan_to_num(transformed_x, False, 0, 0, 0)
         tree.fit(transformed_x, y)
 
         return [tree, intervals, dims, atts]
 
-    def _predict_proba_for_estimator(
-        self, X, classifier, intervals, dims, atts, test_size
-    ):
-        c22 = Catch22()
-
-        transformed_x = np.empty(
-            shape=(self.att_subsample_size * self.__n_intervals, test_size),
-            dtype=np.float32,
-        )
-
-        for j in range(0, self.__n_intervals):
-            for a in range(0, self.att_subsample_size):
-                transformed_x[self.att_subsample_size * j + a] = self.__cif_feature(
-                    X, intervals[j], dims[j], atts[a], c22
-                )
-
-        transformed_x = transformed_x.T
-        np.nan_to_num(transformed_x, False, 0, 0, 0)
-
-        return classifier.predict_proba(transformed_x)
-
-    @staticmethod
-    def __cif_feature(X, intervals, dims, att, c22):
-        if att == 22:
-            # mean
-            return np.mean(X[:, dims, intervals[0] : intervals[1]], axis=1)
-        elif att == 23:
-            # std_dev
-            return np.std(X[:, dims, intervals[0] : intervals[1]], axis=1)
-        elif att == 24:
-            # slope
-            return _slope(X[:, dims, intervals[0] : intervals[1]], axis=1)
+    def _predict_proba_for_estimator(self, X, classifier, intervals, dims, atts):
+        c22 = Catch22(outlier_norm=True)
+        if isinstance(self.tree, ContinuousIntervalTree):
+            return classifier.predict_proba_cif(X, c22, intervals, dims, atts)
         else:
-            return c22._transform_single_feature(
-                X[:, dims, intervals[0] : intervals[1]],
-                feature=att,
+            transformed_x = np.empty(
+                shape=(self.att_subsample_size * self.__n_intervals, X.shape[0]),
+                dtype=np.float32,
             )
+
+            for j in range(0, self.__n_intervals):
+                for a in range(0, self.att_subsample_size):
+                    transformed_x[self.att_subsample_size * j + a] = _cif_feature(
+                        X, intervals[j], dims[j], atts[a], c22
+                    )
+
+            transformed_x = transformed_x.T
+            transformed_x.round(8)
+            np.nan_to_num(transformed_x, False, 0, 0, 0)
+
+            return classifier.predict_proba(transformed_x)
+
+    def temporal_importance_curves(self):
+        if not isinstance(self.tree, ContinuousIntervalTree):
+            raise ValueError(
+                "CIF base estimator for temporal importance curves must"
+                " be ContinuousIntervalTree."
+            )
+
+        curves = np.zeros((25, self.n_dims, self.series_length))
+        for i, tree in enumerate(self.classifiers):
+            splits, gains = tree.tree_splits_gain()
+
+            for n, split in enumerate(splits):
+                gain = gains[n]
+                interval = int(split / self.att_subsample_size)
+                att = self.atts[i][int(split % self.att_subsample_size)]
+                dim = self.dims[i][interval]
+
+                for j in range(
+                    self.intervals[i][interval][0], self.intervals[i][interval][1] + 1
+                ):
+                    curves[att][dim][j] += gain
