@@ -7,30 +7,43 @@ __all__ = ["ESTIMATOR_TEST_PARAMS", "EXCLUDE_ESTIMATORS", "EXCLUDED_TESTS"]
 
 import numpy as np
 
+from sktime.registry import (
+    ESTIMATOR_TAG_LIST,
+    BASE_CLASS_LIST,
+    BASE_CLASS_LOOKUP,
+    TRANSFORMER_MIXIN_LIST,
+)
+
+from pyod.models.knn import KNN
 from hcrystalball.wrappers import HoltSmoothingWrapper
 from sklearn.linear_model import LinearRegression
 from sklearn.pipeline import make_pipeline
 from sklearn.preprocessing import FunctionTransformer
 from sklearn.preprocessing import StandardScaler
-from sktime.forecasting.fbprophet import Prophet
+
+
 from sktime.base import BaseEstimator
-from sktime.classification.base import BaseClassifier
+
+from sktime.annotation.adapters import PyODAnnotator
 from sktime.classification.compose import ColumnEnsembleClassifier
 from sktime.classification.compose import ComposableTimeSeriesForestClassifier
 from sktime.classification.dictionary_based import ContractableBOSS
 from sktime.classification.dictionary_based import TemporalDictionaryEnsemble
+from sktime.classification.hybrid import HIVECOTEV1
 from sktime.classification.interval_based import RandomIntervalSpectralForest
 from sktime.classification.interval_based._cif import CanonicalIntervalForest
 from sktime.classification.interval_based._drcif import DrCIF
 from sktime.classification.interval_based import TimeSeriesForestClassifier as TSFC
 from sktime.classification.interval_based import SupervisedTimeSeriesForest
-from sktime.classification.shapelet_based import ROCKETClassifier
+from sktime.classification.kernel_based import ROCKETClassifier
+from sktime.classification.kernel_based import Arsenal
 from sktime.classification.shapelet_based import ShapeletTransformClassifier
 from sktime.forecasting.arima import AutoARIMA
-from sktime.forecasting.base import BaseForecaster
 from sktime.forecasting.bats import BATS
 from sktime.forecasting.compose import DirectTabularRegressionForecaster
+from sktime.forecasting.compose import DirRecTimeSeriesRegressionForecaster
 from sktime.forecasting.compose import DirectTimeSeriesRegressionForecaster
+from sktime.forecasting.compose import DirRecTabularRegressionForecaster
 from sktime.forecasting.compose import EnsembleForecaster
 from sktime.forecasting.compose import MultioutputTabularRegressionForecaster
 from sktime.forecasting.compose import MultioutputTimeSeriesRegressionForecaster
@@ -38,8 +51,10 @@ from sktime.forecasting.compose import RecursiveTabularRegressionForecaster
 from sktime.forecasting.compose import RecursiveTimeSeriesRegressionForecaster
 from sktime.forecasting.compose import StackingForecaster
 from sktime.forecasting.compose import TransformedTargetForecaster
+from sktime.forecasting.compose import ForecastingPipeline
 from sktime.forecasting.compose import MultiplexForecaster
 from sktime.forecasting.exp_smoothing import ExponentialSmoothing
+from sktime.forecasting.fbprophet import Prophet
 from sktime.forecasting.hcrystalball import HCrystalBallForecaster
 from sktime.forecasting.model_selection import ForecastingGridSearchCV
 from sktime.forecasting.model_selection import ForecastingRandomizedSearchCV
@@ -48,15 +63,9 @@ from sktime.forecasting.naive import NaiveForecaster
 from sktime.forecasting.online_learning import OnlineEnsembleForecaster
 from sktime.forecasting.tbats import TBATS
 from sktime.forecasting.theta import ThetaForecaster
-from sktime.performance_metrics.forecasting import sMAPE
-from sktime.regression.base import BaseRegressor
+from sktime.performance_metrics.forecasting import MeanAbsolutePercentageError
 from sktime.regression.compose import ComposableTimeSeriesForestRegressor
 from sktime.series_as_features.compose import FeatureUnion
-from sktime.transformations.base import BaseTransformer
-from sktime.transformations.base import _PanelToPanelTransformer
-from sktime.transformations.base import _PanelToTabularTransformer
-from sktime.transformations.base import _SeriesToPrimitivesTransformer
-from sktime.transformations.base import _SeriesToSeriesTransformer
 from sktime.transformations.panel.compose import ColumnTransformer
 from sktime.transformations.panel.compose import (
     SeriesToPrimitivesRowTransformer,
@@ -67,6 +76,8 @@ from sktime.transformations.panel.interpolate import TSInterpolator
 from sktime.transformations.panel.reduce import Tabularizer
 from sktime.transformations.panel.shapelets import ContractedShapeletTransform
 from sktime.transformations.panel.shapelets import ShapeletTransform
+from sktime.transformations.panel.signature_based import SignatureTransformer
+from sktime.classification.signature_based import SignatureClassifier
 from sktime.transformations.panel.summarize import FittedParamExtractor
 from sktime.transformations.panel.tsfresh import TSFreshFeatureExtractor
 from sktime.transformations.panel.tsfresh import (
@@ -80,22 +91,39 @@ from sktime.transformations.series.impute import Imputer
 from sktime.transformations.series.compose import OptionalPassthrough
 from sktime.transformations.series.outlier_detection import HampelFilter
 from sktime.transformations.series.boxcox import BoxCoxTransformer
-
+from sktime.dists_kernels.compose_tab_to_panel import AggrDist
+from sktime.dists_kernels.scipy_dist import ScipyDist
 
 # The following estimators currently do not pass all unit tests
 # What do they fail? ShapeDTW fails on 3d_numpy_input test, not set up for that
 EXCLUDE_ESTIMATORS = [
     "ShapeDTW",
-    "HIVECOTEV1",
     "ElasticEnsemble",
     "ProximityForest",
     "ProximityStump",
     "ProximityTree",
 ]
 
+
+# This is temporary until BaseObject is implemented
+DIST_KERNELS_IGNORE_TESTS = [
+    "check_fit_updates_state",
+    "_make_fit_args",
+    "check_fit_returns_self",
+    "check_raises_not_fitted_error",
+    "check_fit_idempotent",
+    "check_fit_does_not_overwrite_hyper_params",
+    "check_methods_do_not_change_state",
+    "check_persistence_via_pickle",
+]
+
+
 EXCLUDED_TESTS = {
     "ShapeletTransformClassifier": ["check_fit_idempotent"],
     "ContractedShapeletTransform": ["check_fit_idempotent"],
+    "HIVECOTEV1": ["check_fit_idempotent", "check_multiprocessing_idempotent"],
+    "ScipyDist": DIST_KERNELS_IGNORE_TESTS,
+    "AggrDist": DIST_KERNELS_IGNORE_TESTS,
 }
 
 # We here configure estimators for basic unit testing, including setting of
@@ -119,6 +147,7 @@ TRANSFORMERS = [
     ),
 ]
 REGRESSOR = LinearRegression()
+ANOMALY_DETECTOR = KNN()
 TIME_SERIES_CLASSIFIER = TSFC(n_estimators=3)
 TIME_SERIES_CLASSIFIERS = [
     ("tsf1", TIME_SERIES_CLASSIFIER),
@@ -126,16 +155,21 @@ TIME_SERIES_CLASSIFIERS = [
 ]
 FORECASTER = ExponentialSmoothing()
 FORECASTERS = [("ses1", FORECASTER), ("ses2", FORECASTER)]
-STEPS = [
+STEPS_y = [
     ("transformer", Detrender(ThetaForecaster())),
     ("forecaster", NaiveForecaster()),
 ]
+STEPS_X = [
+    ("transformer", TabularToSeriesAdaptor(StandardScaler())),
+    ("forecaster", NaiveForecaster()),
+]
 ESTIMATOR_TEST_PARAMS = {
     OnlineEnsembleForecaster: {"forecasters": FORECASTERS},
     FeatureUnion: {"transformer_list": TRANSFORMERS},
     DirectTabularRegressionForecaster: {"estimator": REGRESSOR},
     MultioutputTabularRegressionForecaster: {"estimator": REGRESSOR},
     RecursiveTabularRegressionForecaster: {"estimator": REGRESSOR},
+    DirRecTabularRegressionForecaster: {"estimator": REGRESSOR},
     DirectTimeSeriesRegressionForecaster: {
         "estimator": make_pipeline(Tabularizer(), REGRESSOR)
     },
@@ -145,7 +179,11 @@ ESTIMATOR_TEST_PARAMS = {
     MultioutputTimeSeriesRegressionForecaster: {
         "estimator": make_pipeline(Tabularizer(), REGRESSOR)
     },
-    TransformedTargetForecaster: {"steps": STEPS},
+    DirRecTimeSeriesRegressionForecaster: {
+        "estimator": make_pipeline(Tabularizer(), REGRESSOR)
+    },
+    TransformedTargetForecaster: {"steps": STEPS_y},
+    ForecastingPipeline: {"steps": STEPS_X},
     EnsembleForecaster: {"forecasters": FORECASTERS},
     StackingForecaster: {"forecasters": FORECASTERS, "final_regressor": REGRESSOR},
     Detrender: {"forecaster": FORECASTER},
@@ -153,13 +191,13 @@ ESTIMATOR_TEST_PARAMS = {
         "forecaster": NaiveForecaster(strategy="mean"),
         "cv": SingleWindowSplitter(fh=1),
         "param_grid": {"window_length": [2, 5]},
-        "scoring": sMAPE(),
+        "scoring": MeanAbsolutePercentageError(symmetric=True),
     },
     ForecastingRandomizedSearchCV: {
         "forecaster": NaiveForecaster(strategy="mean"),
         "cv": SingleWindowSplitter(fh=1),
         "param_distributions": {"window_length": [2, 5]},
-        "scoring": sMAPE(),
+        "scoring": MeanAbsolutePercentageError(symmetric=True),
     },
     TabularToSeriesAdaptor: {"transformer": StandardScaler()},
     ColumnEnsembleClassifier: {
@@ -197,14 +235,34 @@ ESTIMATOR_TEST_PARAMS = {
         ],
         "selected_forecaster": "Naive_mean",
     },
-    ShapeletTransformClassifier: {"n_estimators": 3, "time_contract_in_mins": 0.125},
-    ContractedShapeletTransform: {"time_contract_in_mins": 0.125},
+    ShapeletTransformClassifier: {
+        "n_estimators": 3,
+        "transform_contract_in_mins": 0.075,
+    },
+    ContractedShapeletTransform: {"time_contract_in_mins": 0.075},
     ShapeletTransform: {
         "max_shapelets_to_store_per_class": 1,
         "min_shapelet_length": 3,
         "max_shapelet_length": 4,
     },
+    SignatureTransformer: {
+        "augmentation_list": ("basepoint", "addtime"),
+        "depth": 3,
+        "window_name": "global",
+    },
+    SignatureClassifier: {
+        "augmentation_list": ("basepoint", "addtime"),
+        "depth": 3,
+        "window_name": "global",
+    },
     ROCKETClassifier: {"num_kernels": 100},
+    Arsenal: {"num_kernels": 100},
+    HIVECOTEV1: {
+        "stc_params": {"n_estimators": 2, "transform_contract_in_mins": 0.025},
+        "tsf_params": {"n_estimators": 2},
+        "rise_params": {"n_estimators": 2},
+        "cboss_params": {"n_parameter_samples": 6, "max_ensemble_size": 2},
+    },
     TSFreshFeatureExtractor: {"disable_progressbar": True, "show_warnings": False},
     TSFreshRelevantFeatureExtractor: {
         "disable_progressbar": True,
@@ -255,9 +313,16 @@ ESTIMATOR_TEST_PARAMS = {
     AutoCorrelationTransformer: {"n_lags": 1},
     Imputer: {"method": "mean"},
     HampelFilter: {"window_length": 3},
-    OptionalPassthrough: {"transformer": BoxCoxTransformer(), "passthrough": False},
+    OptionalPassthrough: {"transformer": BoxCoxTransformer(), "passthrough": True},
+    AggrDist: {"transformer": ScipyDist()},
+    PyODAnnotator: {"estimator": ANOMALY_DETECTOR},
 }
 
+# We use estimator tags in addition to class hierarchies to further distinguish
+# estimators into different categories. This is useful for defining and running
+# common tests for estimators with the same tags.
+VALID_ESTIMATOR_TAGS = tuple(ESTIMATOR_TAG_LIST)
+
 # These methods should not change the state of the estimator, that is, they should
 # not change fitted parameters or hyper-parameters. They are also the methods that
 # "apply" the fitted estimator to data and useful for checking results.
@@ -269,38 +334,15 @@ NON_STATE_CHANGING_METHODS = (
     "inverse_transform",
 )
 
-# We use estimator tags in addition to class hierarchies to further distinguish
-# estimators into different categories. This is useful for defining and running
-# common tests for estimators with the same tags.
-VALID_ESTIMATOR_TAGS = (
-    "fit-in-transform",  # fitted in transform or non-fittable
-    "univariate-only",
-    "transform-returns-same-time-index",
-    "handles-missing-data",
-)
-
 # The following gives a list of valid estimator base classes.
-VALID_TRANSFORMER_TYPES = (
-    _SeriesToPrimitivesTransformer,
-    _SeriesToSeriesTransformer,
-    _PanelToTabularTransformer,
-    _PanelToPanelTransformer,
-)
-VALID_ESTIMATOR_BASE_TYPES = (
-    BaseClassifier,
-    BaseRegressor,
-    BaseForecaster,
-    BaseTransformer,
-)
+VALID_TRANSFORMER_TYPES = tuple(TRANSFORMER_MIXIN_LIST)
+
+VALID_ESTIMATOR_BASE_TYPES = tuple(BASE_CLASS_LIST)
+
 VALID_ESTIMATOR_TYPES = (
     BaseEstimator,
     *VALID_ESTIMATOR_BASE_TYPES,
     *VALID_TRANSFORMER_TYPES,
 )
 
-VALID_ESTIMATOR_BASE_TYPE_LOOKUP = {
-    "classifier": BaseClassifier,
-    "regressor": BaseRegressor,
-    "forecaster": BaseForecaster,
-    "transformer": BaseTransformer,
-}
+VALID_ESTIMATOR_BASE_TYPE_LOOKUP = BASE_CLASS_LOOKUP
